define([
  'jquery',
  'modernizr/modernizr',
  'matchMedia',
  'domReady!',
  'hoverIntent'
], function($, Modernizr, mediaCheck){ 
  
  //
  // Global Menu Functions
  //
  
  var ua = navigator.userAgent.toLowerCase();
  var isAndroid = ua.indexOf("android") > -1;
  var currentYPos;
  var desktopBreakpoint = 1200;

  function setMenuHeight() {
    var getBodyHeight = document.body.clientHeight;
    var getWindowHeight = window.innerHeight;
    var setHeight;

    if (window.innerWidth < desktopBreakpoint) {

      console.log(getBodyHeight);
      console.log(getWindowHeight);

      if (getBodyHeight >= getWindowHeight) {
        setHeight = getBodyHeight;
      } else {
        setHeight = getWindowHeight;
      }

      document.querySelector('.main-menu').style.height = setHeight + 'px';
      // document.querySelector('.main-menu__inner-list').style.height = setHeight + 'px';
    } else {
      document.querySelector('.main-menu').style.height = '48px';
      // document.querySelector('.main-menu__inner-list').style.height = 'auto';
    }
  }
  
  //
  // Mobile Menu Functions
  //
  
  function toggleMobileMenu() {
    //var getYPos = document.querySelector('html').scrollTop;
  
    if (document.body.classList.contains('show-nav')) {
      document.body.classList.remove('show-nav');
    } else {
      document.body.classList.add('show-nav');
    }

    document.querySelector('[data-action="toggle-nav"]').classList.toggle('open');
    //autoScroll(getYPos);
  }
  
  function toggleMobileSubMenu(e) {
    e.preventDefault();

    var closestLi = this.closest('li')
    
    if (closestLi.classList.contains('show-nav')) {
      closestLi.classList.remove('show-nav');
    } else {
      closestLi.classList.add('show-nav');
    }
  }
  
  function triggerMobileMenu() {

    // need to restrict this to mobile menu only for cross over orientationchange devices
    if (window.innerWidth < desktopBreakpoint) {
      document.querySelector('[data-action="toggle-nav"]').addEventListener('click', toggleMobileMenu);
    }
    
  }
  
  function triggerMobileSubMenu() {
    
    // need to restrict this to mobile menu only for cross over orientationchange devices
    if (window.innerWidth < desktopBreakpoint) {
      var mobileMenuParent = document.querySelectorAll('.main-menu__item--parent > .main-menu__link, .main-menu__inner-item--parent > .main-menu__inner-link');
      
      [].forEach.call(mobileMenuParent, function(parent) {
        
        parent.addEventListener('click', toggleMobileSubMenu);
        
      });
    }
    
  }

  function triggerBackLink() {

    if (window.innerWidth < desktopBreakpoint) {
      var mobileMenuBackLinks = document.querySelectorAll('.main-menu__inner-link--back');
      // messy markup leads to messy js
      [].forEach.call(mobileMenuBackLinks, function(backLink) {

        // if (backLink.closest('.main-menu__inner-item').classList.contains('main-menu__inner-item--level1')) {
          backLink.addEventListener('click', function(){
            this.closest('.main-menu__item--parent').classList.remove('show-nav');
          });
        // } else {
        //   backLink.addEventListener('click', function(){
        //     this.closest('.main-menu__inner-item--parent').classList.remove('show-nav');
        //   });
        // } 
        
      });
    }
  }

  function triggerCloseMenu() {

    if (window.innerWidth < desktopBreakpoint) {
      // This is a list of header elements that are still visible when the main menu is open
      // Clicking on them should close the menu and remove all the necessary classes
      // Add and remove element classes as needed e.g. `.block-search`
      var headerToggles = document.querySelectorAll('.showcart');
      
      [].forEach.call(headerToggles, function(toggle){
        toggle.addEventListener('click', function(){
          document.body.classList.remove('show-nav');
          document.querySelector('[data-action="toggle-nav"]').classList.remove('open');
        });
      });
    }
  }

  function detachMainMenu() {
    var getMainMenu = document.querySelector('.main-menu');
    document.body.insertAdjacentElement('beforeend', getMainMenu);
  }

  // function autoScroll(yPos) {
    
  //   if (document.body.classList.contains('show-nav')) {
  //     document.querySelector('html').scrollTop = 0;
  //     return currentYPos = yPos;
  //   } else {
  //     document.querySelector('html').scrollTop = currentYPos;
  //   }
  // }

  function createBackLink() {
    var getLevel1Menus = document.querySelectorAll('.main-menu__inner-list--level1');
    // var getLevel2Menus = document.querySelectorAll('.main-menu__inner-list--level2');
    var parentTitle;

    [].forEach.call(getLevel1Menus, function(level1) {
      parentTitle = 'Menu';
      var createBackLink = '<li class="main-menu__inner-item main-menu__inner-item--level1"><a class="main-menu__inner-link main-menu__inner-link--back"><span class="title"><span title="back" class="main-menu__icon--back"></span> Back to ' + parentTitle + '</span></a></li>';
      level1.insertAdjacentHTML('afterbegin', createBackLink);
    });

    // [].forEach.call(getLevel2Menus, function(level2) {
    //   var getParentMenuTitle = level2.closest('.main-menu__inner-list--level1').querySelector('.main-menu__inner-item--all .title').textContent;
    //   parentTitle = getParentMenuTitle;
    //   var createBackLink = '<li class="main-menu__inner-item main-menu__inner-item--level2"><a class="main-menu__inner-link main-menu__inner-link--back"><span class="title"><span title="back" class="main-menu__icon--back"></span> Back to ' + parentTitle + '</span></a></li>';
    //   level2.insertAdjacentHTML('afterbegin', createBackLink);
    // });
  }

  function removeHoverTrigger() {
    $('.main-menu__item--parent').unbind("mouseenter").unbind("mouseleave").removeProp('hoverIntent_t').removeProp('hoverIntent_s');
  }
  
  //
  // Desktop Menu Functions
  //

  var menuScreen = document.querySelector('.main-menu__screen');

  function attachMainMenu() {
    var getMainMenu = document.querySelector('.main-menu');

    // This may need to be changed if the referenceContainer has been changed in the XML
    var getHeaderContentBlock = document.querySelector('.page-header');
    //

    getHeaderContentBlock.insertAdjacentElement('afterend', getMainMenu);
  }

  function destroyBackLink() {
    var getBackLinks = document.querySelectorAll('.main-menu__inner-link--back');

    if (getBackLinks != undefined || getBackLinks != null) {

      [].forEach.call(getBackLinks, function(backLink) {
        backLink.parentElement.remove();
      });
    }
  }

  function createUnderlay() {

    if (document.querySelector('.menu-underlay') == undefined || document.querySelector('.menu-underlay') == null) {
      var menuUnderlay = '<div class="menu-underlay"></div>';
      document.body.insertAdjacentHTML('afterBegin', menuUnderlay);
      
      if (window.Modernizr.touchevents) {
        document.querySelector('.menu-underlay').addEventListener('click', function(){
          document.body.classList.remove('menu-hovered');
          destroyUnderlay();
        })
      }
    }
  }

  function destroyUnderlay() {
    if (document.querySelector('.menu-underlay') != undefined || document.querySelector('.menu-underlay') != null) {
      document.querySelector('.menu-underlay').remove();
    }
  }

  function setScreenHeight(sub) {
    var getSubMenuHeight = sub.getBoundingClientRect().height;
    menuScreen.style.height = getSubMenuHeight + 'px';  
  }

  function showHoverMenu() {
    var thisSub = this.querySelector('.main-menu__inner-list--level1');

    document.body.classList.add('menu-hovered');
    this.classList.add('item-hovered');
    setScreenHeight(thisSub);
    createUnderlay();
  }

  function removeHoverMenu() {
    this.classList.remove('item-hovered');
  }

  function triggerHoverMenu() {
    $('.main-menu .main-menu__item--parent').hoverIntent({
      over: showHoverMenu,
      out: removeHoverMenu,
      timeout: 100
    });

    document.querySelector('.main-menu').addEventListener('mouseleave', function() {
      document.body.classList.remove('menu-hovered');
      destroyUnderlay();
    });

    var getChildlessItems = document.querySelectorAll('.main-menu__item:not(.main-menu__item--parent)');
    
    [].forEach.call(getChildlessItems, function(childless){
      childless.addEventListener('mouseover', function(){
        document.body.classList.remove('menu-hovered');
        destroyUnderlay();
      });
    });
    
  }

  function preventDefaultOnTouchDevices() {
  
    // need to restrict this to desktop menu only for cross over orientationchange devices
    if (window.innerWidth > (desktopBreakpoint - 1)) {
      var getTopLevelLinks = document.querySelectorAll('.main-menu__item--parent > .main-menu__link');

      [].forEach.call(getTopLevelLinks, function(link) {
        link.addEventListener('click', function preventFirstClick(e){
          e.preventDefault();
          link.removeEventListener('click', preventFirstClick); // allow the link to fire on second click/tap
        });
      });
    }
    
  }


  //
  // The business end
  //

  mediaCheck({
    media: '(min-width: ' + desktopBreakpoint + 'px)',

    entry: function() {
      console.log('desktop menu');
      
      setTimeout(function(){
        // create, destroy, movement of elements, setting of heights, etc
        destroyBackLink();
        attachMainMenu();
        setMenuHeight();

        // attach events when dom is done
        triggerHoverMenu();

        if (window.Modernizr.touchevents && isAndroid === true) {
          preventDefaultOnTouchDevices();
        }
      }, 300);
    },

    exit: function() {
      console.log('mobile menu');
      
      setTimeout(function(){
        // creation and movement of elements, setting of heights, etc
        createBackLink();
        detachMainMenu();
        setMenuHeight();

        // detach desktop events
        removeHoverTrigger();

        // attach mobile events
        triggerMobileMenu();
        triggerMobileSubMenu();
        triggerBackLink();
        triggerCloseMenu();
      }, 300);
    }
  });
  
});